"""
MAUS web application URL handler functions.
"""

#  This file is part of MAUS: http://micewww.pp.rl.ac.uk:8080/projects/maus
#
#  MAUS is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MAUS is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with MAUS.  If not, see <http://www.gnu.org/licenses/>.

import logging
import os
import os.path
import random
import tempfile

from django.core.files.storage import default_storage
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.views.decorators.cache import cache_control

from mausweb.settings import MEDIA_ROOT
from mausweb.maus.image_utils import convert_image
from mausweb.maus.image_utils import thumb_image
from mausweb.maus.image_utils import create_http_response_image
from mausweb.maus.image_utils import get_raw_path
from mausweb.maus.image_utils import get_reco_path
from mausweb.maus.image_utils import is_image
from mausweb.maus.image_utils import parse_path

@cache_control(no_cache=True)
def index(request): # pylint: disable=W0613
    """
    List the images in the default storage directory's "reco"
    sub-directory and, from these, build the index.html web page.
    Valid image file types are listed in image_utils.IMAGE_TYPES.
    @param request HTTP request.
    @return HTTP response.
    """
    files = default_storage.listdir("reco")[1] 
    file_names = []
    for afile in files:
        [_, file_type] = parse_path(afile)
        if is_image(file_type):
            file_names.append(afile)
        file_list = list(file_names)
        file_list.sort()
    # Pass image list to templates/maus/index.html.
    return render_to_response('maus/index.html', {'images': file_list})

def images(request): # pylint: disable=W0613
    """
    List the images in the default storage directory's "reco"
    sub-directory and, from these, build the index.html web page.
    Valid image file types are listed in image_utils.IMAGE_TYPES.
    Only those that match a search string given in a "keyword"
    value in the request are shown.
    @param request HTTP request.
    @return HTTP response.
    """
    keyword = request.GET.get('keyword', '')
    logger = logging.getLogger("django")
    if (logger.isEnabledFor(logging.DEBUG)):
        logger.debug("Keyword: %s" % keyword)
    files = default_storage.listdir("reco")[1] 
    matches = []
    for afile in files:
        if (afile.find(keyword) != -1):
            [_, file_type] = parse_path(afile)
            if is_image(file_type):
                # Use random to prevent browser caching images.
                matches.append([afile, file_type, random.random()])
    # Pass image list to templates/maus/images.html.
    return render_to_response('maus/images.html', {'images': matches})

def reco(request, image): # pylint: disable=W0613
    """
    Return an HTTP response that consists of an image in the default
    storage directory's "reco" sub-directory. 
    @param request HTTP request.
    @param image Image name.
    @return HTTP response.
    @throws Http404 if the image is not found.
    """
    image_path = get_reco_path(image)
    image_file = default_storage.open(image_path)
    data = image_file.read()
    image_file.close()
    [_, image_type] = parse_path(image)
    return HttpResponse(data, content_type="image/%s" % image_type)

def raw(request, image): # pylint: disable=W0613
    """
    Return an HTTP response that consists of an image in the default
    storage directory's "raw" sub-directory. 
    @param request HTTP request.
    @param image Image name.
    @return HTTP response.
    @throws Http404 if the image is not found.
    """
    image_path = get_raw_path(image)
    image_file = default_storage.open(image_path)
    data = image_file.read()
    image_file.close()
    [_, image_type] = parse_path(image)
    return HttpResponse(data, content_type="image/%s" % image_type)

def graph(request, image): # pylint: disable=W0613
    """
    Return an HTTP response that consists of an image in the default
    storage directory's "reco" sub-directory embedded within a
    graph.html web page.
    @param request HTTP request.
    @param image Image name.
    @return HTTP response.
    @throws Http404 if the image is not found.
    """
    get_reco_path(image) # For validation only.
    [image_name, image_type] = parse_path(image)
    # Pass image name and type to templates/maus/graph.html.
    return render_to_response('maus/graph.html', 
        {'image_name': image_name, 'image_type': image_type})

def thumb(request, image): # pylint: disable=W0613
    """
    Return an HTTP response that consists of an image in the default
    storage directory's "reco" sub-directory converted into a JPG 
    thumbnail. The thumbnail is cached into a "thumbs" sub-directory. 
    @param request HTTP request.
    @param image Image name.
    @return HTTP response.
    @throws Http404 if the image is not found.
    """
    image_path = get_reco_path(image)
    [image_name, _] = parse_path(image)
    absolute_image_path = os.path.join(MEDIA_ROOT, image_path)
    # Create thumbnail name.
    thumb_name = image_name + "-thumb" + ".jpg"
    absolute_thumb_path = os.path.join(MEDIA_ROOT, "thumbs", thumb_name)
    # Convert image to JPG via MagickWand for sharper thumbnails.
    logger = logging.getLogger("django")
    if (logger.isEnabledFor(logging.DEBUG)):
        logger.debug("Thumbs: converting %s" % image)
    thumb_image(absolute_image_path, "jpg", absolute_thumb_path, 100)
    # Save thumbnail data to HTTP response.
    return create_http_response_image(absolute_thumb_path, "jpeg")

def convert(image, target_image_type):
    """
    Convert an image in the default storage directory's "reco"
    sub-directory into an image of the requested type and return
    an HTTP response with this image.
    @param image Image name.
    @param target_image_type Requested image type.
    @return HTTP response.
    @throws Http404 if the image is not found.
    """
    image_path = get_reco_path(image)
    [_, image_type] = parse_path(image)
    absolute_image_path = os.path.join(MEDIA_ROOT, image_path)
    tmp_path = None
    # Convert image if it is not already of the target type, using
    # a temporary file to save converted image.
    if (image_type != target_image_type):
        [_, tmp_path] = tempfile.mkstemp(suffix=".%s" % target_image_type)
        convert_image(absolute_image_path, target_image_type, tmp_path)
        absolute_image_path = tmp_path
    # Save image data to HTTP response.
    response = \
        create_http_response_image(absolute_image_path, target_image_type)
    # Clean up.
    if (tmp_path != None):
        os.remove(tmp_path)
    return response 

def png(request, image): # pylint: disable=W0613
    """
    Return an HTTP response that consists of an image in the default
    storage directory's "reco" sub-directory converted to a PNG.
    @param request HTTP request.
    @param image Image name.
    @return HTTP response.
    @throws Http404 if the image is not found.
    """
    return convert(image, "png")

def eps(request, image): # pylint: disable=W0613
    """
    Return an HTTP response that consists of an image in the default
    storage directory's "reco" sub-directory converted to EPS.
    @param request HTTP request.
    @param image Image name.
    @return HTTP response.
    @throws Http404 if the image is not found.
    """
    return convert(image, "eps")
